@@ -122,7 +122,7 @@ int main( int argc, char **argv ) {
                     mem[meminfo_main][meminfo_shared] >> byteshift,
                     mem[meminfo_main][meminfo_buffers] >> byteshift,
                     mem[meminfo_main][meminfo_cached] >> byteshift);
-            printf("-/+ buffers/cache: %10d %10d\n", 
+	    if (!old_fmt) printf("-/+ buffers/cache: %10d %10d\n", 
                    (mem[meminfo_main][meminfo_used]
                     -mem[meminfo_main][meminfo_buffers]
 		    -mem[meminfo_main][meminfo_cached])>> byteshift,
