@@ -5,8 +5,6 @@
 *	license, version 2, or any later version.  See file COPYING	*
 *	for information on distribution conditions.			*
 \***********************************************************************/
-
-
 #include <stdlib.h>
 #include <stdio.h>
 
@@ -22,4 +20,30 @@ void *xcalloc(void *pointer, int size) {
     }
 }
 
+void *xmalloc(unsigned int size) {
+    void *p;
 
+    if (size == 0)
+        ++size;
+    p = malloc(size);
+    if (!p) {
+	fprintf(stderr, "xmalloc: malloc(%d) failed", size);
+	perror(NULL);
+	exit(1);
+    }
+    return(p);
+}
+
+void *xrealloc(void *oldp, unsigned int size) {
+    void *p;
+
+    if (size == 0)
+        ++size;
+    p = realloc(oldp, size);
+    if (!p) {
+	fprintf(stderr, "xrealloc: realloc(%d) failed", size);
+	perror(NULL);
+	exit(1);
+    }
+    return(p);
+}
