@@ -77,7 +77,7 @@ void stat2proc(char* S, proc_t* P) {
     sscanf(S, "%d (%39c", &P->pid, P->cmd);
     sscanf(tmp + 2,			/* skip space after ')' too */
 	   "%c %d %d %d %d %d %lu %lu %lu %lu %lu %ld %ld %ld %ld %d "
-	   "%d %lu %lu %ld %lu %lu %lu %lu %lu %lu %lu %lu %d %d %d %d %lu",
+	   "%d %lu %lu %ld %lu %lu %lu %lu %lu %lu %lu %lu %LX %LX %LX %LX %lu",
 	   &P->state, &P->ppid, &P->pgrp, &P->session, &P->tty, &P->tpgid,
 	   &P->flags, &P->min_flt, &P->cmin_flt, &P->maj_flt, &P->cmaj_flt,
 	   &P->utime, &P->stime, &P->cutime, &P->cstime, &P->priority, &P->nice,
@@ -310,8 +310,17 @@ proc_t* readproctree(int flags, ...) {
     
     /* pass through apropriate arguments to openproc */
     va_start(ap, flags);
-    if (Do(UID))
-	PT = openproc(flags, va_arg(ap, uid_t*), va_arg(ap, int));
+    if (Do(UID)) {
+	/* temporary variables to ensure that va_arg() instances
+	 * are called in the right order
+	 */
+	uid_t* u;
+	int i;
+
+	u = va_arg(ap, uid_t*);
+	i = va_arg(ap, int);
+	PT = openproc(flags, u, i);
+    }
     else if (Do(PID) || Do(TTY) || Do(STAT))
 	PT = openproc(flags, va_arg(ap, void*));
     else
@@ -336,6 +345,7 @@ proc_t* readproctree(int flags, ...) {
 	    tmp2->r = tmp->l->r;		/* insert as child of found node */
 	    tmp->l->r = node;
 	}
+    closeproc(PT);
     return &tree;
 }
 
@@ -351,8 +361,17 @@ proc_t** readproctab(int flags, ...) {
     va_list ap;
 
     va_start(ap, flags);		/* pass through args to openproc */
-    if (Do(UID))
-	PT = openproc(flags, va_arg(ap, uid_t*), va_arg(ap, int));
+    if (Do(UID)) {
+	/* temporary variables to ensure that va_arg() instances
+	 * are called in the right order
+	 */
+	uid_t* u;
+	int i;
+
+	u = va_arg(ap, uid_t*);
+	i = va_arg(ap, int);
+	PT = openproc(flags, u, i);
+    }
     else if (Do(PID) || Do(TTY) || Do(STAT))
 	PT = openproc(flags, va_arg(ap, void*)); /* assume ptr sizes same */
     else
@@ -362,6 +381,7 @@ proc_t** readproctab(int flags, ...) {
 	tab = xrealloc(tab, (n+1)*sizeof(proc_t*));/* realloc as we go, using */
 	tab[n] = readproc(PT, NULL);		  /* final null to terminate */
     } while (tab[n++]);				  /* stop when NULL reached */
+    closeproc(PT);
     return tab;
 }
 
