@@ -39,6 +39,7 @@
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/resource.h>
+#include <sys/param.h>
 
 void show_short(char*, proc_t*);
 void show_long (char*, proc_t*);
@@ -56,12 +57,12 @@ struct {
     const char*	default_sort;
     const char*	header;
 } mode[] = {
-    { show_short,  0 , "Up", "  PID TTY STAT  TIME COMMAND" },
+    { show_short,  0 , "Up", "  PID TTY STAT TIME COMMAND" },
     { show_long,  'l', "Pp", " FLAGS   UID   PID  PPID PRI  NI   SIZE   RSS WCHAN       STA TTY TIME COMMAND" },
     { show_user,  'u', "up", "USER       PID %CPU %MEM  SIZE   RSS TTY STAT START   TIME COMMAND" },
-    { show_jobs,  'j', "gPp"," PPID   PID  PGID   SID TTY TPGID  STAT   UID   TIME COMMAND" },
+    { show_jobs,  'j', "gPp"," PPID   PID  PGID   SID TTY TPGID  STAT  UID   TIME COMMAND" },
     { show_sig,   's', "p",  "  UID   PID SIGNAL   BLOCKED  IGNORED  CATCHED  STAT TTY   TIME COMMAND" },
-    { show_vm,    'v', "r",  "  PID TTY STAT  TIME  PAGEIN TSIZ DSIZ  RSS   LIM %MEM COMMAND" },
+    { show_vm,    'v', "r",  "  PID TTY STAT TIME  PAGEIN TSIZ DSIZ  RSS   LIM %MEM COMMAND" },
     { show_m,     'm', "r",  "  PID TTY MAJFLT MINFLT   TRS   DRS  SIZE  SWAP   RSS  SHRD   LIB  DT COMMAND" },
     { show_regs,  'X', "p",  "NR   PID    STACK      ESP      EIP TMOUT ALARM STAT TTY   TIME COMMAND" },
     { NULL, 0, NULL, NULL }
@@ -154,9 +155,11 @@ int main(int argc, char **argv) {
                     } else if (*(p+2) != '\0')	/* just '-- '; not an error */
 			usage("ps: unknown long option\n");
 		}
-		fprintf(stderr,
-			"warning: `-' deprecated; use `ps %s', not `ps %s'\n",
-			(p+1), p);
+		if (!getenv("I_WANT_A_BROKEN_PS")) {
+		    fprintf(stderr,
+			    "warning: `-' deprecated; use `ps %s', not `ps %s'\n",
+			    (p+1), p);
+		}
 		break;
 	      case 'l': CL_fmt = PS_L;   /* replaceable by a */	break;
 	      case 'u': CL_fmt = PS_U;   /* loop over mode[] */	break;
@@ -266,13 +269,13 @@ int main(int argc, char **argv) {
 void usage(char* context) {
     fprintf(stderr,
 	    "%s"
-            "usage:  ps -acehjlnrsSuvwx{t<tty>|#|O[-]u[-]U..} \\\n"
+            "usage:  ps acehjlnrsSuvwx{t<tty>|#|O[-]u[-]U..} \\\n"
             "           --sort:[-]key1,[-]key2,...\n"
             "           --help gives you this message\n"
             "           --version prints version information\n",
 	    context ? context : "");
     if (context)
-	exit(1);	/* prevent bad exit status by calling usage("") */
+	exit(1);	/* prevent bad exit status by calling usage(NULL) */
 }
 
 /* set maximum chars displayed on a line based on screen size.
@@ -311,7 +314,14 @@ void show_procs(unsigned maxcmd, int do_header, int pflags, void* args, int N) {
     while ((next = readproc(tab,retbuf))) {	/* read next process */
 	n++;					/* Now either: */
 	if (CL_forest) {			/*    add process to tree */
-	    static char s[256];
+	    /* FIXME: this static is disgusting, but to keep binary
+	     * compatibility within the 1.2 series I'll just go from
+	     * 256 bytes to 1024 bytes, which should be good enough
+	     * for most people....  :-(  In the future, the format
+	     * format function should probably dynamically allocate
+	     * strings.
+	     */
+	    static char s[1024];
 	    if (CL_num_outp)
 		snprintf(next->ttyc, sizeof next->ttyc, "%04.4x", next->tty);
 	    (mode[CL_fmt].format)(s, next);
@@ -407,7 +417,7 @@ void show_jobs(char *s, proc_t *p) {
 }
 
 void show_user(char *s, proc_t *p) {
-    int pmem, total_time, seconds;
+    long pmem, total_time, seconds;
     time_t start;
     unsigned int pcpu;
 
@@ -415,7 +425,9 @@ void show_user(char *s, proc_t *p) {
 	s += sprintf(s, "%5d    ", p->uid);
     else
 	s += sprintf(s, "%-8s ", p->user);
-    seconds = (((GL_current_time * HZ) - p->start_time) / HZ);
+    /* the (long) coerces the calculation to the 64-bit domain on
+       64-bit machines */
+    seconds = (((GL_current_time * (long)HZ) - p->start_time) / HZ);
     start = GL_time_now - seconds;
     total_time = (p->utime + p->stime +
 		  (CL_Sum ? p->cutime + p->cstime : 0));
@@ -424,14 +436,14 @@ void show_user(char *s, proc_t *p) {
 	0;
     if (pcpu > 999) pcpu = 999;
     pmem = p->rss * 1000 / (GL_main_mem >> PAGE_SHIFT);
-    sprintf(s, "%5d %2u.%u %2d.%d %5ld %5ld %2s %s%.6s ",
+    sprintf(s, "%5d %2u.%u %2ld.%ld %5ld %5ld %2s %s %.6s ",
 	    p->pid,  pcpu / 10, pcpu % 10,  pmem / 10, pmem % 10,
 	    p->vsize >> 10, p->rss << (PAGE_SHIFT - 10), p->ttyc, status(p),
 	    ctime(&start) + (GL_time_now - start > 3600*24 ? 4 : 10));
 }
 
 void show_sig(char *s, proc_t *p) {
-    sprintf(s, "%5d %5d %08x %08x %08x %08x %s %3s ",
+    sprintf(s, "%5d %5d %08Lx %08Lx %08Lx %08Lx %s  %3s ",
 	    p->uid, p->pid, p->signal, p->blocked, p->sigignore, p->sigcatch,
 	    status(p), p->ttyc);
 }
@@ -474,7 +486,7 @@ void show_m(char *s, proc_t *p) {
 void show_regs(char *s, proc_t *p) {
     char time1[16], time2[16];
 
-    sprintf(s, "%2ld %5d %8lx %8lx %8lx %s %s %s %3s ",
+    sprintf(s, "%2ld %5d %8lx %8lx %8lx %s %s %s  %3s ",
 	    p->start_code >> 26, p->pid, p->start_stack,
 	    p->kstk_esp, p->kstk_eip,
 	    prtime(time1, p->timeout, GL_current_time*HZ),
