@@ -56,6 +56,12 @@
  * Modified 1996/06/25 Zygo Blaxell <zblaxell@ultratech.net>
  * Added field scaling code for programs that run more than two hours or
  * take up more than 100 megs.  We have lots of both on our production line.
+ *
+ * Modified 1998/02/21 Kirk Bauer <kirk@kaybee.org>
+ * Added the 'u' option to display only a selected user... plus it will
+ * take into account that not all 20 top processes are actually shown,
+ * so it can fit more onto the screen.  I think this may help the
+ * 'don't show idle' mode, but I'm not sure.
  */
 
 #include <errno.h>
@@ -76,6 +82,7 @@
 #include <ctype.h>
 #include <setjmp.h>
 #include <stdarg.h>
+#include <sys/param.h>
 
 /* need NR_TASKS */
 #include <linux/tasks.h>
@@ -222,8 +229,8 @@ void get_options(void)
 	strcpy(Fields, DEFAULT_SHOW);
     } else {
 	if (fgets(Fields, 254, fp) != NULL) {
-	    pt = strstr(Fields, "\n");
-	    *pt = 0;
+	    pt = strchr(Fields, '\n');
+	    if (pt) *pt = 0;
 	}
 	fgets(Options, 254, fp);
 	fclose(fp);
@@ -294,6 +301,8 @@ int main(int argc, char **argv)
     /* The key read in. */
     char c;
 
+    struct sigaction sact;
+
     get_options();
     /*
      * Parse arguments.
@@ -370,15 +379,21 @@ int main(int argc, char **argv)
     }
 
     setup_terminal();
-    window_size();
+    window_size(0);
     /*
      * Set up signal handlers.
      */
-    signal(SIGHUP, (void *) (int) end);
-    signal(SIGINT, (void *) (int) end);
-    signal(SIGQUIT, (void *) (int) end);
-    signal(SIGTSTP, (void *) (int) stop);
-    signal(SIGWINCH, (void *) (int) window_size);
+    sact.sa_handler = end;
+    sact.sa_flags = 0;
+    sigemptyset(&sact.sa_mask);
+    sigaction(SIGHUP, &sact, NULL);
+    sigaction(SIGINT, &sact, NULL);
+    sigaction(SIGQUIT, &sact, NULL);
+    sact.sa_handler = stop;
+    sigaction(SIGTSTP, &sact, NULL);
+    sact.sa_handler = window_size;
+    sact.sa_flags = SA_RESTART;
+    sigaction(SIGWINCH, &sact, NULL);
 
     /* loop, collecting process info and sleeping */
     while (1) {
@@ -419,7 +434,7 @@ void error_end(int rno)
 /*
 	 * Normal end of execution.
 	 */
-void end(void)
+void end(int signo)
 {
     if (psdbsucc)
 	close_psdb();
@@ -432,7 +447,7 @@ void end(void)
 /*
 	 * SIGTSTP catcher.
 	 */
-void stop(void)
+void stop(int signo)
 {
     /* Reset terminal. */
     if (psdbsucc)
@@ -440,10 +455,9 @@ void stop(void)
     ioctl(0, TCSETSF, &Savetty);
     PUTP(tgoto(cm, 0, Lines - 3));
     fflush(stdout);
-    raise(SIGTSTP);
+    raise(SIGSTOP);
     /* Later... */
     ioctl(0, TCSETSF, &Rawtty);
-    signal(SIGTSTP, (void *) (int) stop);
     longjmp(redraw_jmp, 1);
 }
 
@@ -452,7 +466,7 @@ void stop(void)
        * and also catches SIGWINCHs, and adjusts Maxlines.  Basically, this is
        * the central place for window size stuff.
        */
-void window_size(void)
+void window_size(int signo)
 {
     struct winsize ws;
 
@@ -468,7 +482,6 @@ void window_size(void)
      * calculate header size, length of cmdline field ...
      */
     Numfields = make_header();
-    signal(SIGWINCH, (void *) (int) window_size);
 }
 /*
        * this adjusts the lines needed for the header to the current value
@@ -985,6 +998,7 @@ void show_procs(void)
     static proc_t **p_table=NULL;
     static int proc_flags;
     int count;
+    int ActualLines;
     float elapsed_time;
     unsigned int main_mem;
     static int first=0;
@@ -1025,19 +1039,22 @@ void show_procs(void)
      * Lather, rinse, repeat.
      */
     count = 0;
-    while ((count < Maxlines) && (p_table[count]->pid!=-1)) {
+    ActualLines = 0;
+    while ((ActualLines < Maxlines) && (p_table[count]->pid!=-1)) {
 	int pmem;
 	char stat;
 
 	stat = p_table[count]->state;
 
-	if (!Noidle || (stat != 'S' && stat != 'Z')) {
-
+	if ( (!Noidle || (stat != 'S' && stat != 'Z')) &&
+	     ( (CurrUser[0] == '\0') ||
+	       (!strcmp(CurrUser,p_table[count]->user) ) ) ) {
 	    /*
 	     * Show task info.
 	     */
 	    pmem = p_table[count]->resident * 1000 / (main_mem / 4096);
 	    show_task_info(p_table[count], pmem);
+	    ActualLines++;
 	}
 	count++;
     }
@@ -1189,7 +1206,7 @@ void do_stats(proc_t** p, float elapsed_time, int pass)
 	n++;
 	if (n > NR_TASKS) {
 	    printf(PROGNAME ": Help!  Too many tasks!\n");
-	    end();
+	    end(0);
 	}
     }
 
@@ -1244,7 +1261,7 @@ void do_key(char c)
      * First the commands which don't require a terminal mode switch.
      */
     if (c == 'q')
-	end();
+	end(0);
     else if (c == ' ')
         return;
     else if (c == 12) {
@@ -1280,7 +1297,7 @@ void do_key(char c)
 	fputs(Noidle ? "on" : "off", stdout);
 	PUTP(me);
 	fputs("\n\n", stdout);
-	printf("%s\n\nPress any key to continue\n", Secure ? SECURE_HELP_SCREEN : HELP_SCREEN);
+	printf("%s\n\nPress any key to continue", Secure ? SECURE_HELP_SCREEN : HELP_SCREEN);
 	ioctl(0, TCSETS, &Rawtty);
 	(void) getchar();
 	break;
@@ -1288,6 +1305,10 @@ void do_key(char c)
 	Noidle = !Noidle;
 	SHOWMESSAGE(("No-idle mode %s", Noidle ? "on" : "off"));
 	break;
+      case 'u':
+	SHOWMESSAGE(("Which User (Blank for All): "));
+	strcpy(CurrUser,getstr());
+	break;
       case 'k':
 	if (Secure)
 	    SHOWMESSAGE(("\aCan't kill in secure mode"));
@@ -1343,7 +1364,7 @@ void do_key(char c)
 	numinput = getint();
 	if (numinput != -1) {
 	    Display_procs = numinput;
-	    window_size();
+	    window_size(0);
 	}
 	break;
       case 'r':
@@ -1487,8 +1508,17 @@ proc_t** readproctab2(int flags, proc_t** tab, ...) {
     va_list ap;
 
     va_start(ap, tab);		/* pass through args to openproc */
-    if (Do(UID))
-	PT = openproc(flags, va_arg(ap, uid_t*), va_arg(ap, int));
+    if (Do(UID)) {
+	/* temporary variables to ensure that va_arg() instances
+	 * are called in the right order
+	 */
+	uid_t* u;
+	int i;
+
+	u = va_arg(ap, uid_t*);
+	i = va_arg(ap, int);
+	PT = openproc(flags, u, i);
+    }
     else if (Do(PID) || Do(TTY) || Do(STAT))
 	PT = openproc(flags, va_arg(ap, void*)); /* assume ptr sizes same */
     else
