@@ -1,5 +1,5 @@
 /* w - show what logged in users are doing.  Almost entirely rewritten from
- * scratch by Charles Blake circal June 1996.  Some vestigal traces of the
+ * scratch by Charles Blake circa June 1996.  Some vestigal traces of the
  * original may exist.  That was done in 1993 by Larry Greenfield with some
  * fixes by Michael K. Johnson.
  */
@@ -31,7 +31,7 @@ void    showinfo(utmp_t *login_rec, int formtype, int maxcmd, int from);
 proc_t *getproc(utmp_t *login_rec, char *tty, int *jcpu, int *bogus);
 void    print_logintime(time_t login_time, FILE* fout);
 
-#ifndef W_SHOWFROM
+#ifdef W_SHOWFROM
 #   define FROM_STRING "on"
 #else
 #   define FROM_STRING "off"
@@ -46,7 +46,7 @@ int main(int argc, char **argv) {
     struct stat sbuf;
     struct winsize win;
 
-#ifdef W_SHOWFROM
+#ifndef W_SHOWFROM
     from = 0;
 #endif
     set_linux_version();
@@ -89,18 +89,18 @@ int main(int argc, char **argv) {
 	print_uptime();
 	printf("USER     TTY      ");
 	if (from)
-	    printf("FROM             ");
+	    printf("FROM            ");
 	if (longform)
-	    printf(" LOGIN@  IDLE   JCPU   PCPU  WHAT\n");
+	    printf("  LOGIN@   IDLE   JCPU   PCPU  WHAT\n");
 	else
-	    printf("  IDLE   WHAT\n");
+	    printf("   IDLE  WHAT\n");
     }
 
     utmpname(UTMP_FILE);
     setutent();
     while ((u=getutent())) {
  	if (u->ut_type == USER_PROCESS &&
- 	    (user ? strncmp(u->ut_user, user, USERSZ) : *u->ut_user))
+ 	    (user ? !strncmp(u->ut_user, user, USERSZ) : *u->ut_user))
  	    showinfo(u, longform, maxcmd, from);
     }
     endutent();
@@ -116,8 +116,13 @@ void print_host(char* host, int len) {
     char *last;
     int width = 0;
 
-    last = host + len - 1;
-    for ( ; host <= last ; host++)
+    /* FIXME: there should really be a way to configure this... */
+    /* for now, we'll just limit it to the 16 that the libc5 version
+     * of utmp uses.
+     */
+    if (len > 16) len = 16;
+    last = host + len;
+    for ( ; host < last ; host++)
         if (isprint(*host) && *host != ' ') {
 	    fputc(*host, stdout);
 	    ++width;
@@ -174,10 +179,10 @@ void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
     if (formtype) {
 	printf("%-9.8s%-9.8s", uname, u->ut_line);
 	if (from)
-	    print_host(u->ut_host, sizeof u->ut_host + 1);
+	    print_host(u->ut_host, sizeof u->ut_host);
 	print_logintime(u->ut_time, stdout);
 	if (*u->ut_line == ':')			/* idle unknown for xdm logins */
-	    printf("       ");
+	    printf("   ?   ");
 	else
 	    print_time_ival7(idletime(u, tty), 0, stdout);
 	print_time_ival7(jcpu/HZ, (jcpu%HZ)*(100./HZ), stdout);
@@ -189,7 +194,11 @@ void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
     } else {
 	printf("%-9.8s%-9.8s", u->ut_user, u->ut_line);
 	if (from)
-	    print_host(u->ut_host, sizeof u->ut_host + 1);
+	    print_host(u->ut_host, sizeof u->ut_host);
+	if (*u->ut_line == ':')			/* idle unknown for xdm logins */
+	    printf("   ?   ");
+	else
+	    print_time_ival7(idletime(u, tty), 0, stdout);
     }
     fputs("  ", stdout);
     if (best)
@@ -224,12 +233,12 @@ void print_logintime(time_t logt, FILE* fout) {
     if (hour == 0)  hour = 12;
     if (curt - logt > 12*60*60 && logtm->tm_yday != today)
 	if (curt - logt > 6*24*60*60)
-	    fprintf(fout, "%2d%3s%2d", logtm->tm_mday, month[logtm->tm_mon],
+	    fprintf(fout, " %2d%3s%2d", logtm->tm_mday, month[logtm->tm_mon],
 		    logtm->tm_year % 100);
 	else
-	    fprintf(fout, "%3s%2d%s", weekday[logtm->tm_wday], hour, merid);
+	    fprintf(fout, " %3s%2d%s", weekday[logtm->tm_wday], hour, merid);
     else
-	fprintf(fout, "%2d:%02d%s", hour, logtm->tm_min, merid);
+	fprintf(fout, " %2d:%02d%s", hour, logtm->tm_min, merid);
 }
 
 /* This function scans the process table accumulating total cpu times for
