@@ -2,7 +2,7 @@
 *  Displays CPU state distribution
 */
 #ifndef lint
-static char rcsid[] = "$Header: /ai/lambda/X.V11R4/contrib/clients/xcpustate/RCS/xcpustate.c,v 1.2 90/02/20 00:33:36 xwindows Exp $";
+static char rcsid[] = "$Header: /usr/rhs/CVS/procps/xcpustate/xcpustate.c,v 1.1 1997/09/02 16:03:06 johnsonm Exp $";
 #endif /*lint*/
 
 #include <stdio.h>
@@ -28,9 +28,9 @@ static int count, interval;
 /* Application Resources - no particular widget */
 static XtResource application_resources[] = {
     {"interval", "Interval", XtRInt, sizeof(int),
-	    (Cardinal)&interval, XtRInt, (caddr_t) &defaultInterval},
+	    0, XtRInt, (caddr_t) &defaultInterval},
     {"count", "Count", XtRInt, sizeof(int),
-	    (Cardinal)&count, XtRInt, (caddr_t) &defaultCount},
+	    0, XtRInt, (caddr_t) &defaultCount},
 };
 
 /*
@@ -147,8 +147,13 @@ char **argv;
 
     if (argc > 1)
 	usage();
-	
-    XtGetApplicationResources(topLevel, 0, application_resources,
+
+    application_resources[0].resource_offset = 
+      (Cardinal)&interval - (Cardinal)&application_resources;
+    application_resources[1].resource_offset = 
+      (Cardinal)&count - (Cardinal)&application_resources;
+
+    XtGetApplicationResources(topLevel, application_resources, application_resources,
 			      XtNumber(application_resources), NULL, 0 );
 
     form = XtCreateManagedWidget("form", formWidgetClass, topLevel,
